package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.CopyOnWriteArrayList;

import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;

/**
 * Diese Klasse liefert einen Taktgeber. ber einen Flankenwechsel kann man sich
 * durch einen Listener informieren lassen.
 * 
 * @author HofmannJ
 * 
 */
public class Taktgeber implements IWorkbenchListener
{
	/**
	 * Singleton-Instanz dieser Klasse.
	 */
	private static Taktgeber _instance = null;

	/**
	 * Takt in Millisekunden.
	 */
	private static final long _invervall = 1000l;

	/**
	 * Sichert intern den aktuellen Zustand der Taktflanke.
	 */
	private boolean _flanke = false;

	/**
	 * Timert zur Generierung des Taktes.
	 */
	private Timer _timer;

	/**
	 * In dieser Liste werden smtliche Taktlistener gefhrt.
	 */
	private CopyOnWriteArrayList<ITaktListener> _lTaktListener = new CopyOnWriteArrayList<ITaktListener>();

	/**
	 * Privater Konstruktor, fhrt die Initialisierung des Taktgebers durch.
	 */
	private Taktgeber() {
		startTaktgeber();

		if (PlatformUI.getWorkbench() != null)
		{
			PlatformUI.getWorkbench().addWorkbenchListener(this);
		}
	}

	/**
	 * Liefert die Singleton-Instanz dieser Klasse.
	 * 
	 * @return Singleton-Instanz dieser Klasse.
	 */
	public static Taktgeber getInstance()
	{
		if (_instance == null)
		{
			synchronized (Taktgeber.class)
			{
				if (_instance == null)
				{
					_instance = new Taktgeber();
				}
			}
		}

		return _instance;
	}

	/**
	 * Startet den Taktgeber.
	 */
	private void startTaktgeber()
	{
		if (_timer == null)
		{
			TimerTask task = new TimerTask()
			{

				@Override
				public void run()
				{
					_flanke = (!_flanke);

					Iterator<ITaktListener> it = _lTaktListener.iterator();
					while (it.hasNext())
					{
						ITaktListener taktListener = it.next();
						if (taktListener != null)
						{
							taktListener.takt(_flanke);
						}
					}
				}
			};

			_timer = new Timer("TimerTask Taktgeber @ " + _invervall
					+ " Millisekunden");
			_timer.schedule(task, 0l, _invervall);
		}
	}

	/**
	 * Stoppt den Taktgeber.
	 */
	private void stopTaktgeber()
	{
		if (_timer != null)
		{
			_timer.cancel();
			_timer = null;
		}
	}

	/**
	 * Fgt einen {@link ITaktListener} hinzu, welcher ab sofort ber die
	 * steigende bzw. fallende Taktflanke informiert wird.
	 * 
	 * @param taktListener
	 *          Hinzuzufgender {@link ITaktListener}.
	 */
	public void addTaktListener(ITaktListener taktListener)
	{
		if ((taktListener != null) && (!_lTaktListener.contains(taktListener)))
		{
			_lTaktListener.add(taktListener);
		}
	}

	/**
	 * Entfernt einen {@link ITaktListener}, dieser wird ab sofort nicht mehr ber
	 * die steigende bzw. fallende Taktflanke informiert.
	 * 
	 * @param taktListener
	 */
	public void removeTaktListener(ITaktListener taktListener)
	{
		if ((taktListener != null) && (_lTaktListener.contains(taktListener)))
		{
			_lTaktListener.remove(taktListener);
		}
	}

	public void removeTaktListeners(Collection<ITaktListener> listeners)
	{
		if ((listeners != null) && (listeners.size() > 0))
		{
			for (ITaktListener taktListener : listeners)
			{
				_lTaktListener.remove(taktListener);
			}
		}
	}

	public static void main(String[] args)
	{
		Taktgeber taktgeber = Taktgeber.getInstance();
		ITaktListener listener = new ITaktListener()
		{

			@Override
			public void takt(boolean taktflanke)
			{
				System.out.println("Taktflanke => " + taktflanke);
			}
		};
		taktgeber.addTaktListener(listener);
	}

	@Override
	public void postShutdown(IWorkbench workbench)
	{
		/*
		 * Sobald die Workbench gestoppt ist, muss der Taktgeber beendet werden.
		 */
		stopTaktgeber();
	}

	@Override
	public boolean preShutdown(IWorkbench workbench, boolean forced)
	{
		/*
		 * Es soll der Workbench immer erlaubt sein, sich zu beenden.
		 */
		return true;
	}

	/**
	 * Liefert alle angemeldeten {@link ITaktListener} in einem Array zurck.
	 * 
	 * @return Array aller angemeldeten {@link ITaktListener}.
	 */
	public ITaktListener[] getAngemeldeteTaktlistener()
	{
		LinkedList<ITaktListener> listeners = new LinkedList<ITaktListener>();
		listeners.addAll(_lTaktListener);
		return listeners.toArray(new ITaktListener[listeners.size()]);
	}

}
