package de.bsvrz.sba.plugin.actions.parameter;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Registrierung fr Parameter vom Typ {@link IParameter}.
 * 
 * @author HofmannJ
 * 
 */
public class ParameterRegistry
{
	/**
	 * Speichert die {@link IParameter}
	 */
	private Map<String, IParameter> _hmParameter = null;

	/**
	 * Konstruktor, fhrt die Initialisierung durch. Die Struktur ist hiernach
	 * leer
	 */
	public ParameterRegistry()
	{
		initialize();
	}

	/**
	 * Konstruktor, fhrt die Initialisierung durch. bernimmt eine Liste von
	 * {@link IParameter}. Die Struktur ist hiernach mit den aus der Liste
	 * bernommenen {@link IParameter}n gefllt.
	 * 
	 * @param parameters
	 *          Liste von {@link IParameter}s zur Initialisierung der Struktur.
	 */
	public ParameterRegistry(Collection<IParameter> parameters)
	{
		initialize();

		setParameters(parameters);
	}

	/**
	 * Fhrt die Initalisierung der Map durch
	 */
	private void initialize()
	{
		_hmParameter = Collections
				.synchronizedMap(new HashMap<String, IParameter>());
	}

	/**
	 * bernimmt eine Liste von {@link IParameter}s. Diese werden in der Map
	 * gesetzt. Evtl. vorhandene Werte werden durch die neuen ersetzt.
	 * 
	 * @param parameters
	 *          Liste der zu setzenden {@link IParameter}.
	 */
	public void setParameters(Collection<IParameter> parameters)
	{
		Iterator<IParameter> it = parameters.iterator();
		while (it.hasNext())
		{
			IParameter parameter = it.next();
			setParameter(parameter);
		}
	}

	/**
	 * Setzt einen {@link IParameter} in der Map. Ein evtl. vorhandener
	 * {@link IParameter} mit der gleichen ID wird ersetzt.
	 * 
	 * @param parameter
	 *          Zu setzender {@link IParameter}.
	 */
	public void setParameter(IParameter parameter)
	{
		_hmParameter.put(parameter.getId(), parameter);
	}

	/**
	 * Liefert den Wert eines {@link IParameter} als String zurck.
	 * 
	 * @param id
	 *          ID des Parameters.
	 * @return String des Wertes des {@link IParameter}s oder <code>null</code>,
	 *         wenn dieser {@link IParameter} nicht vorhanden ist.
	 */
	public String getParameterValueAsString(String id)
	{
		String re = null;

		if (_hmParameter.containsKey(id))
		{
			IParameter parameter = _hmParameter.get(id);

			if (parameter.getValue() instanceof String)
			{
				re = (String) parameter.getValue();
			}
		}

		return re;
	}

	/**
	 * Liefert den short-Wert des {@link IParameter}
	 * 
	 * @param id
	 *          ID des {@link IParameter}
	 * @return Liefert den Wert des {@link IParameter} als short oder
	 *         <code>-1</code>, wenn nicht vorhanden.
	 */
	public short getParameterValueAsShort(String id)
	{
		short re = -1;

		String parameter = getParameterValueAsString(id);
		if (parameter != null)
		{
			try
			{
				re = Short.valueOf(parameter);
			}
			catch (NumberFormatException e)
			{
				e.printStackTrace();
			}
		}

		return re;
	}

	/**
	 * Liefert den Wert des {@link IParameter} als int.
	 * 
	 * @param id
	 *          ID des Parameters.
	 * @return Wert des {@link IParameter} oder <code>-1</code>, falls nicht
	 *         vorhanden.
	 */
	public int getParameterValueAsInt(String id)
	{
		int re = -1;

		String parameter = getParameterValueAsString(id);
		if (parameter != null)
		{
			try
			{
				re = Integer.valueOf(parameter);
			}
			catch (NumberFormatException e)
			{
				e.printStackTrace();
			}
		}

		return re;
	}

	/**
	 * Liefert den Wert des {@link IParameter} als long.
	 * 
	 * @param id
	 *          Id des {@link IParameter}
	 * @return Wert des {@link IParameter} als long oder <code>-1</code>, falls
	 *         nicht vorhanden.
	 */
	public long getParameterValueAsLong(String id)
	{
		long re = -1;

		String parameter = getParameterValueAsString(id);
		if (parameter != null)
		{
			try
			{
				re = Long.valueOf(parameter);
			}
			catch (NumberFormatException e)
			{
				e.printStackTrace();
			}
		}

		return re;
	}

	/**
	 * Liefert das Objekt des Wertes des {@link IParameter} zurck.
	 * 
	 * @param id
	 *          ID des {@link IParameter}
	 * @return {@link Object} des {@link IParameter} oder <code>null</code>,
	 *         wenn der {@link IParameter} nicht existiert oder der Wert nicht
	 *         gesetzt ist.
	 */
	public Object getParameterValueAsObject(String id)
	{
		Object re = null;

		if (_hmParameter.containsKey(id))
		{
			re = _hmParameter.get(id).getValue();
		}

		return re;
	}

	/**
	 * Liefert den Parameter, sofern vorhanden, zurck.
	 * 
	 * @param id
	 *          ID des zu liefernden Parameters.
	 * @return Liefert den Parameter oder <code>null</code>, wenn der Parameter
	 *         nicht existiert.
	 */
	public IParameter getParameter(String id)
	{
		IParameter re = null;

		if (_hmParameter.containsKey(id))
		{
			re = _hmParameter.get(id);
		}

		return re;
	}

	/**
	 * Liefert eine Liste aller {@link IParameter} dieser Registrierung zurck.
	 * 
	 * @return Liste aller {@link IParameter} dieser Registrierung. Die Liste ist
	 *         leer, wenn kein {@link IParameter} registriert wurde.
	 */
	public LinkedList<IParameter> getParameters()
	{
		LinkedList<IParameter> re = new LinkedList<IParameter>();

		Iterator<Entry<String, IParameter>> it = _hmParameter.entrySet().iterator();
		while (it.hasNext())
		{
			re.add(it.next().getValue());
		}

		return re;
	}

	/**
	 * Liefert eine Liste der IDs aller {@link IParameter} dieser Registrierung
	 * zurck.
	 * 
	 * @return Liste aller IDs
	 */
	public LinkedList<String> getParameterIds()
	{
		LinkedList<String> re = new LinkedList<String>();

		Iterator<Entry<String, IParameter>> it = _hmParameter.entrySet().iterator();
		while (it.hasNext())
		{
			re.add(it.next().getKey());
		}

		return re;
	}
	
	/**
	 * Setzt die Map zurck. Alle Parameter werden entfernt.
	 */
	public void clearParameters()
	{
		if(_hmParameter != null)
		{
			_hmParameter.clear();
		}
	}

}
