package de.bsvrz.sba.plugin.actions.parameter.verwaltung;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;

import de.bsvrz.sba.plugin.actions.Activator;
import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;

/**
 * Diese Klasse ist fr die Verwaltung des Erweiterungspunktes
 * <code>de.bsvrz.sba.plugin.action.parameterAction</code> verwantwortlich.
 * 
 * @author HofmannJ
 * 
 */
public class ParameterActionRegistry
{
	/**
	 * ID des Erweiterungspunktes
	 */
	private static final String IDEXT = "de.bsvrz.sba.plugin.action.parameterAction";

	/**
	 * Liste der {@link ParameterActionRegistryElement}, welche den
	 * Erweiterungspunkt <code>de.bsvrz.sba.plugin.action.parameterAction</code>
	 * reprsentieren.
	 */
	private LinkedList<ParameterActionRegistryElement> _elemente = new LinkedList<ParameterActionRegistryElement>();

	/**
	 * Konstruktor. bernimmt einmalig das einlesen der Erweiterungspunkte vom Typ
	 * <code>de.bsvrz.sba.plugin.action.parameterAction</code>, wenn eine
	 * {@link IParameterAction} im Parameter <code>class</code> und Attribute
	 * vom Typ <code>userId</code> beim Benutzen des Erweiterungspunktes
	 * definiert wurden.
	 */
	private ParameterActionRegistry()
	{
		IConfigurationElement[] config = Platform.getExtensionRegistry()
				.getConfigurationElementsFor(IDEXT);

		for (IConfigurationElement ce : config)
		{
			String contributor = ce.getContributor().getName();

			LinkedList<String> ids = new LinkedList<String>();
			IParameterAction parameterAction = null;

			String[] attributeNames = ce.getAttributeNames();
			for (String attName : attributeNames)
			{
				if (attName.equals("class"))
				{
					try
					{
						Object object = ce.createExecutableExtension("class");

						if (object instanceof IParameterAction)
						{
							parameterAction = (IParameterAction) object;

							IConfigurationElement[] children = ce.getChildren();
							for (IConfigurationElement child : children)
							{
								String[] childAttNames = child.getAttributeNames();
								for (String childAttName : childAttNames)
								{
									if (childAttName.equals("userId"))
									{
										ids.add(child.getAttribute("userId"));
									}
								}
							}
						}

					}
					catch (CoreException e)
					{
						e.printStackTrace();
					}
				}
			}

			if ((ids.size() > 0) && (parameterAction != null))
			{
				ParameterActionRegistryElement pve = new ParameterActionRegistryElement(
						parameterAction, ids, contributor);

				/*
				 * Prfen, ob schon eine Registrierung einer IParameterAction unter der
				 * gleichen ID vom gleichen Contributor stattgefunden hat. Nur wenn dies
				 * nicht der Fall ist, wird die Action hinzugefgt.
				 */
				boolean nochNichtVorhanden = true;
				Iterator<ParameterActionRegistryElement> itEle = _elemente.iterator();
				while (itEle.hasNext())
				{
					ParameterActionRegistryElement element = itEle.next();
					IParameterAction action = element.getParameterAction();
					if (action != null)
					{
						if ((parameterAction.getParameterActionId().equals(action
								.getParameterActionId()))
								&& (pve.getContributor().equals(element.getContributor())))
						{
							{
								String msg = "";
								msg = msg
										+ "Es wurde Versucht die IParameterAction mit der ID \""
										+ pve.getParameterAction().getParameterActionId() + "\""
										+ " durch Contributor \"" + contributor + "\""
										+ " mehrfach zu registrieren. Dies ist nicht gestattet.";

//								MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
//										.getDisplay().getActiveShell(), SWT.OK | SWT.ICON_ERROR);
//								mb.setText("Doppelte Registrierung nicht erstattet!");
//								mb.setMessage(msg);
//								mb.open();
								
								Activator.getDefault().getLog().log(
										new Status(Status.WARNING, Activator.PLUGIN_ID, msg));

								nochNichtVorhanden = false;
								break;
							}
						}
					}
				}

				if (nochNichtVorhanden)
				{
					_elemente.add(pve);
				}
			}

		}
	}

	/**
	 * Speichert die Singleton-Instanz der Klasse {@link ParameterActionRegistry}.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private static class Inner
	{
		private static ParameterActionRegistry INSTANCE = new ParameterActionRegistry();
	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link ParameterActionRegistry}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link ParameterActionRegistry}
	 */
	public static ParameterActionRegistry getInstance()
	{
		return Inner.INSTANCE;
	}

	/**
	 * Liefert eine Liste der {@link ParameterActionRegistryElement}e.
	 * 
	 * @return Liste der {@link ParameterActionRegistryElement}e
	 */
	public LinkedList<ParameterActionRegistryElement> getParameterActionElements()
	{
		return _elemente;
	}

	/**
	 * Liefert eine Liste derjenigen {@link ParameterActionRegistryElement}e die
	 * von sich behaupten, dass sie von einem User mit einer bestimmten ID
	 * verwendet werden knnen.
	 * 
	 * @param userId
	 *          UserID, zu dem zugehrige {@link ParameterActionRegistryElement}e
	 *          gefunden werden sollen.
	 * @return Liste der gefundenen {@link ParameterActionRegistryElement}e. Die
	 *         Liste ist leer, wenn keine {@link ParameterActionRegistryElement}e
	 *         gefunden wurden.
	 */
	public LinkedList<ParameterActionRegistryElement> getParameterActionElements(
			String userId)
	{
		LinkedList<ParameterActionRegistryElement> re = new LinkedList<ParameterActionRegistryElement>();
		Iterator<ParameterActionRegistryElement> itVerwaltungsElemente = _elemente
				.iterator();

		while (itVerwaltungsElemente.hasNext())
		{
			ParameterActionRegistryElement verwaltungsElement = itVerwaltungsElemente
					.next();
			Iterator<String> itUserIds = verwaltungsElement.getUserIds().iterator();
			while (itUserIds.hasNext())
			{
				String id = itUserIds.next();
				if (userId.equals(id))
				{
					re.add(verwaltungsElement);
					break;
				}
			}
		}

		return re;
	}
}
