/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Bestimmt smtliche, dynamische Objekttypen des aktuellen DaVs.
 * 
 * @author HofmannJ
 * 
 */
public class DynamicObjectTypesDetermination
{
	/**
	 * Supertyp, welcher den dynamischen Objekttyp definiert. An Hand der Subtypen
	 * knnen weiter dynamische Objekttypen bestimmt werden.
	 */
	private static final String S_DYN_OBJ_SUPER_TYPE = "typ.dynamischesObjekt";

	/**
	 * HashMap zur Speicherung des dynamischen Objekttypen. Pid als Schlssel und
	 * {@link SystemObjectType} als Wert.
	 */
	private HashMap<String, SystemObjectType> _hmDynObjTypes = new HashMap<String, SystemObjectType>();

	/**
	 * Datenmodell des DaV
	 */
	private DataModel _model = null;

	public DynamicObjectTypesDetermination()
	{
	}

	/**
	 * Bestimmt eine Liste der PIDs aller im DaV bekannten, dynamischen
	 * Objekttypen.
	 * 
	 * @return Liste der PIDs der dynamischen Objekttypen. Ist <code>null</code>,
	 *         wenn keine Verbindung zum DaV aktiv ist oder die dynamischen
	 *         Objekttypen nicht bestimmt werden konnten.
	 */
	public LinkedList<SystemObjectType> determineDynObjTypes()
	{
		_hmDynObjTypes.clear();

		LinkedList<SystemObjectType> re = null;

		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			_model = DaVVerbindung.getInstanz().getDaVVerbindung().getDataModel();
			SystemObjectType superType = _model.getType(S_DYN_OBJ_SUPER_TYPE);
			_hmDynObjTypes.put(superType.getPid(), superType);
			List<SystemObjectType> subTypes = superType.getSubTypes();
			determineDynObjTypesRecursive(subTypes);
		}

		if ((_hmDynObjTypes != null) && (_hmDynObjTypes.size() > 0))
		{
			re = new LinkedList<SystemObjectType>();
			Iterator<Entry<String, SystemObjectType>> it = _hmDynObjTypes.entrySet()
					.iterator();
			while (it.hasNext())
			{
				Entry<String, SystemObjectType> entry = it.next();
				re.add(entry.getValue());
			}
		}

		return re;
	}

	/**
	 * Bestimmt rekursiv die nachfolgenden Subtypen, welche dynamische Objekttypen
	 * sind.
	 * 
	 * @param subTypes
	 *          Liste der auszuwertenden Subtypen.
	 */
	private void determineDynObjTypesRecursive(List<SystemObjectType> subTypes)
	{
		Iterator<SystemObjectType> it = subTypes.iterator();
		while (it.hasNext())
		{
			SystemObjectType objectType = it.next();

			/*
			 * Bestimmen der dynmischen Objekttypen
			 */
			if (!_hmDynObjTypes.containsKey(objectType.getPid()))
			{
				_hmDynObjTypes.put(objectType.getPid(), objectType);
			}

			/*
			 * Rekursive Bestimmung der nachfolgenden, dynmsichen Objekttypen
			 */
			List<SystemObjectType> subSubTypes = objectType.getSubTypes();
			if (subSubTypes.size() > 0)
			{
				determineDynObjTypesRecursive(subSubTypes);
			}
		}
	}

}
