/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verriegelung.spezielleVerriegelungsMatrix;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;

/**
 * Klasse enthlt die Parameter fr die Regelklasse "Verriegelungsmatrix", Regel "Spezielle Verriegelungsmatrix".<br>
 * Regel:<br>
 * An speziellen Anzeigequerschnitten muss die Kombination von bestimmten Inhalten an bestimmten Anzeigen untersagt werden knnen. 
 * Hierzu muss eine Liste von Anzeigen mit dazugehrigen Inhalten parametriert werden. Wenn an allen Anzeigen die entsprechenden 
 * Inhalte aktuell angefordert werden, so ist die Schaltung von der Verriegelung zu unterbinden.<br>
 * Diese Regelklasse muss Regeln enthalten, die in der folgenden Form aufgebaut sind:<br>
 * Wenn  fr alle i gilt: Anzeigeinhalt [Anzeigei] = "Xi" <br>
 * dann an allen Vorgngern und Nachfolgern der Anzeige in Querrichtung (ber beliebig viele Ebenen) 
 * alle Anforderungen als "entfernt" kennzeichnen und eine Betriebsmeldung absetzen. 
 * @version $Revision: 1.2 $ / $Date: 2013/06/27 13:36:26 $ / ($Author: ObertM $)
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 *
 */
public class VerwaltungParaSpezielleVerriegelungsMatrix
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaSpezielleVerriegelungsMatrix _instance;

  /**
   * Debug Ausgabe
   */
  private boolean _debug = false;
  
  /**
   * Pid der angeforderten Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaSpezielleVerriegelungsMatrix";
  
  /**
   * Allgemeiner Schaltgrund fr die Regelklasse
   */
  private Aspect m_schaltgrundAllgemein;
  
  /**
   * Parametersatz pro Anzeigequerschnitt
   */
  private HashMap<SystemObject, ListeMitRegeln> m_parameterProAnzeigeQuerschnitt = new HashMap<SystemObject, ListeMitRegeln>();
  
  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport m_pcs = new PropertyChangeSupport (this);

  /**
   * Mutex fr den synchronisierten Zugriff auf die Datenstruktur
   */
  private Object m_mutex = new Object();

  /**
   * Pid der SBA
   */
  private String _pidSBA;

  /**
   * Pid des Defaultaspekts
   */
  public static String _pidDefaultAspekt = "asp.sswSbaVerriegelung";

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaSpezielleVerriegelungsMatrix ()
  {
  	_pidSBA = Konfig.getInstance().getPidSBA();
    
    m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaSpezielleVerriegelungsMatrix getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaSpezielleVerriegelungsMatrix.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaSpezielleVerriegelungsMatrix ();
        }
      }
    }
    
    return _instance;
  }

//  /**
//   * Methode liefert den zu verwendenden Parametersatz.
//   * @return zu verwendenen Parametersatz fr die Anzeige
//   */
//  public ListeMitRegeln getParameter ()
//  {
//    ListeMitRegeln l = new ListeMitRegeln();
//    
//    synchronized (m_mutex)
//    {
//      l = m_parameterAnlage;
//    }
//      
//    return l;
//  }

  /**
   * Methode liefert den Schaltgrund fr die Regelklasse
   * @return Schaltgrund fr die Regelklasse
   */
  public Aspect getSchaltgrundAllgemein()
  {
    return m_schaltgrundAllgemein;
  }

  /**
   * Methode setzt des Schaltgrund fr dei Regelklasse
   * @param schaltgrund Schaltgrund fr dei Regelklasse
   */
  public void setSchaltgrundAllgemein( Aspect schaltgrund )
  {
    m_schaltgrundAllgemein = schaltgrund;
  }
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.addPropertyChangeListener( pcl );    
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    m_pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    m_pcs.firePropertyChange( "Aktualisierung", null, null );
    
    if (_debug)
      System.out.println(toString());
  }

  

  public void auswerteAttributgruppen (ResultData[] results)
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
      {
        auswerteAttributgruppe (dat);
        
        benachrichtigeListener();
      }
    }
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe 'atg.sswSbaAllgemeineVerriegelungsMatrix'
   * @param dat empfangender Datensatz
   */
  private void auswerteAttributgruppe( ResultData dat )
  {
    synchronized (m_mutex)
    {
      if (!dat.hasData())
      {
        m_schaltgrundAllgemein = null;
        m_parameterProAnzeigeQuerschnitt.clear();
      }
      else
      {
        Data data = dat.getData();
          
        m_schaltgrundAllgemein = (Aspect) data.getReferenceValue( "SchaltgrundAllgemein" ).getSystemObject();
        auswerteAttributgruppeAnlage( data );
      }
      
      m_schaltgrundAllgemein = Konfig.getInstance().getAspekt(_pidDefaultAspekt);
    }
  }

  /**
   * Methode zum Auswerten der fr die gesamte Anlage geltenden Parameter
   * @param data empfangender Datensatz
   */
  private void auswerteAttributgruppeAnlage (Data data)
  {
  	ListeMitRegeln tmp = new ListeMitRegeln();
    tmp.auswerteAttributgruppe( data, getSchaltgrundAllgemein() );
    
    m_parameterProAnzeigeQuerschnitt.clear();
    
    /*
     * Aufteilen auf die einzelnen Anzeigequerschnitte
     */
    
    Vector<RegelSpezielleVerriegelungsMatrix> p = tmp.getParameter();
    for (RegelSpezielleVerriegelungsMatrix rsvm : p)
    {
    	List<Verriegelung> verriegelungen = rsvm.getVerriegelungen();
    	for (Verriegelung v : verriegelungen)
    	{
        SystemObject anzeige = v.getAnzeige();
        SystemObject aq = Konfig.getInstance().getAnzeigeQuerschnittZuAnzeige(anzeige);
        
        if (aq != null)
        {
        	if (!m_parameterProAnzeigeQuerschnitt.containsKey(aq))
        		m_parameterProAnzeigeQuerschnitt.put(aq, new ListeMitRegeln());
        	
        	m_parameterProAnzeigeQuerschnitt.get(aq).addRegel(rsvm);
        	break;
        }
    	}
    }
  }

//  /**
//   * Methode fgt eine Regel fr die allgemeinen Parameter der Anlage hinzu
//   * @param verriegelungen zu prfende Verriegelungen
//   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
//   */
//  public void addRegelAllgemein( List<Verriegelung> verriegelungen, Aspect schaltgrund )
//  {
//    ListeMitRegeln regeln = m_parameterAnlage;
//    regeln.addRegel( verriegelungen, schaltgrund );
//  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    s += "Spezielle Verriegelungsmatrix\n";
    s += "=============================\n";
    
    s += "Parameter Anlage:\n";
    s += "=================\n";
    s += m_parameterProAnzeigeQuerschnitt.toString();
    
    return s;
  }

  /**
   * Methode zum Erzeugen einer Attributgruppe mit den Parametern
   * @return Attributgruppe mit den Parametern
   */
  public Data erzeugeAttributgruppe ()
  {
	  ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection == null)
      return null;

    String pidAtg = _pidAtg;
    
    // Attributgruppe zusammensetzen und versenden
      
    AttributeGroup atg = (AttributeGroup) connection.getDataModel().getAttributeGroup(pidAtg);
    Data data = connection.createData(atg );

    data.getReferenceValue( "SchaltgrundAllgemein" ).setSystemObject( getSchaltgrundAllgemein() );
    
    Array anlageRegeln = data.getArray( "AnlageRegeln" );
    erzeugeAnlageRegeln (anlageRegeln);

    return data;
  }

  /**
   * Methode schreibt die Anlageregeln in ein Array einer Attributgruppe
   * @param array Array fr die Regeln
   */
  private void erzeugeAnlageRegeln( Array array )
  {
    erzeugeRegeln( array, m_parameterProAnzeigeQuerschnitt );
  }

  /**
   * Methode schreibt die Regeln pro Anzeigetyp in ein Array
   * @param array Array in das die Regeln geschrieben werden solle
   * @param parameter Regeln
   */
  private void erzeugeRegeln( Array array, HashMap<SystemObject, ListeMitRegeln> parameter )
  {
  	int anzahl = 0;
  	for (ListeMitRegeln lmr : parameter.values())
  		anzahl += lmr.getParameter().size();

    array.setLength( anzahl );

    int index = 0;
  	for (ListeMitRegeln lmr : parameter.values())
  	{
	    Vector<RegelSpezielleVerriegelungsMatrix> rsvm = lmr.getParameter();
	    
	    for (int i=0; i<rsvm.size(); ++i)
	    {
	      Data aktuell = array.getItem( index++ );
	      RegelSpezielleVerriegelungsMatrix rvm = rsvm.get( i );
	      
	      aktuell.getReferenceValue( "SchaltgrundRegel" ).setSystemObject( rvm.getSchaltgrundRegel() );
	      
	      List<Verriegelung> verriegelungen = rvm.getVerriegelungen();
	      
	      Array av = aktuell.getArray( "Verriegelungen" );
	      av.setLength( verriegelungen.size() );
	      
	      for (int j=0; j<verriegelungen.size(); ++j)
	      {
	        Data d = av.getItem( j );
	        Verriegelung v = verriegelungen.get( j );
	        
	        d.getReferenceValue( "Anzeige" ).setSystemObject( v.getAnzeige() );
	        v.getWvzInhalt().erzeugeAttributgruppe( d.getItem( "Zeichenfolge" ));
	      }
	    }
  	}
  }

  /**
   * Methode fgt Parameter fr einen Anzeigequerschnitt hinzu
   * @param anzeigeQuerschnitt Anzeigequerschnitt
   * @param parameter parameter
   */
  public void addParameter ( SystemObject anzeigeQuerschnitt, ListeMitRegeln parameter )
  {
  	synchronized (m_parameterProAnzeigeQuerschnitt)
    {
  		m_parameterProAnzeigeQuerschnitt.put(anzeigeQuerschnitt, parameter); 
    }
  }

  /**
   * Methode liefert die Parameter als Attributgruppe fr den DaV
   * @return Attributgruppe
   */
	public Data getData()
  {
	  return erzeugeAttributgruppe();
  }
	
	/**
	 * Methode liefert die Parameter fr einen Anzeigequerschnitt
	 * @param anzeigeQuerschnitt Anzeigequerschnitt
	 * @return Parameter fr diesen Anzeigequerschnitt sofern welche versorgt sind, sonst null
	 */
  ListeMitRegeln getParameter (SystemObject anzeigeQuerschnitt)
  {
  	if (m_parameterProAnzeigeQuerschnitt.containsKey(anzeigeQuerschnitt))
  		return m_parameterProAnzeigeQuerschnitt.get(anzeigeQuerschnitt);

    return null;
  }

  /**
   * Methode liefert die Parameter pro Anzeigequerschnitt
   * @return Parameter pro Anzeigequerschnitt
   */
	public HashMap<SystemObject, ListeMitRegeln> getParameterProAnzeigeQuerschnitt()
  {
	  return m_parameterProAnzeigeQuerschnitt;
  }
}
