/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Hilfsklasse zum Empfangen und Senden von Daten an bzw. vom DaV.
 * 
 * @author HofmannJ
 * 
 */
public class DaVKommunikation implements ClientReceiverInterface,
		ClientSenderInterface
{
	/**
	 * Debug-Flag
	 */
	private static final boolean _debug = true;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _connection = null;

	/**
	 * Flag fr den Empfangs-Thread
	 */
	private boolean _laufeEmpfangsThread = false;

	/**
	 * Synchronisations-Objekt fr den Empfangs-Thread
	 */
	private Object _syncEmpfangThread = new Object();

	/**
	 * Synchronisations-Objekt fr den besttigten Empfang
	 */
	private Object _syncEmpfangBeendet = new Object();

	/**
	 * Pid des {@link SystemObject}s
	 */
	private String _pidSystemObjekt = null;

	/**
	 * Pid der {@link AttributeGroup}
	 */
	private String _pidAtg = null;

	/**
	 * Pid des {@link Aspect}s.
	 */
	private String _pidAsp = null;

	/**
	 * Simulationsvariante
	 */
	private short _simVar = 0;

	/**
	 * {@link SystemObject}
	 */
	private SystemObject _systemObject = null;

	/**
	 * {@link DataDescription}
	 */
	private DataDescription _dataDescription = null;

	/**
	 * Empfangene Daten
	 */
	private Data _dataEmpfang = null;

	/**
	 * Objekt zum Synchronisieren an Hand der Sendesteuerung.
	 */
	private Object _sendeSteuerung = new Object();

	/**
	 * Timeout fr das Warten auf die Sendesteuerung
	 */
	private static final long _sendeSteuerungTimeOut = 15000;

	private Data data;

	/**
	 * Konstruktor, bernimmt die Verbindung zum DaV
	 * 
	 * @param connection
	 */
	public DaVKommunikation(ClientDavConnection connection)
	{
		this._connection = connection;
	}

	/**
	 * Empfngt Daten fr den durch eine {@link ParameterEditorKapsel} definierte
	 * Struktur.
	 * 
	 * @param kapsel
	 *          {@link ParameterEditorKapsel}, welche die zu empfangenden Daten
	 *          beschreibt.
	 * @return Liefert die empfangenen Daten zurck oder <code>null</code>,
	 *         wenn keine Daten empfangen wurden.
	 */
	protected Data empfangeDaten(ParameterEditorKapsel kapsel)
	{
		/*
		 * Setzen von Pid des SystemObjektes, der Attributgrzo
		 */
		this._pidSystemObjekt = kapsel.getPidSystemObjekt();
		this._pidAtg = kapsel.getPidAtg();
		this._pidAsp = kapsel.getPidAspEmpfang();
		this._simVar = kapsel.getSimulationsVariante();

		DataModel dataModel = _connection.getDataModel();

		_systemObject = dataModel.getObject(_pidSystemObjekt);
		AttributeGroup _atg = dataModel.getAttributeGroup(_pidAtg);
		Aspect _asp = dataModel.getAspect(_pidAsp);

		_dataDescription = new DataDescription(_atg, _asp, _simVar);

		ResultData resultData = _connection.getData(_systemObject,
				_dataDescription, 300L);
		data = resultData.getData();
		
		if (data == null){
		  data = _connection.createData(_atg);
		}
		
		return data;

		// return empfangeDaten(kapsel.getPidSystemObjekt(), kapsel.getPidAtg(),
		// kapsel.getPidAspEmpfang(), kapsel.getSimulationsVariante());
	}

	/**
	 * Empfngt die durch die Parameter bezeichneten Daten aus dem DaV.
	 * 
	 * @param pidSystemObject
	 *          Pid des {@link SystemObject}s, auf welches die Anmeldung erfolgen
	 *          soll.
	 * @param pidAtg
	 *          Pid der {@link AttributeGroup}, unter welcher die Anmeldung
	 *          erfolgen soll.
	 * @param pidAsp
	 *          Pid des {@link Aspect}s, unter dem die Daten empfangen werden
	 *          sollen.
	 * @param simVar
	 *          Simulationsvariante.
	 * @return Liefert die empfangenen Daten zurck oder <code>null</code>,
	 *         wenn keine Daten empfangen wurden.
	 */
	protected Data empfangeDaten(String pidSystemObject, String pidAtg,
			String pidAsp, short simVar)
	{
		_dataEmpfang = null;

		if (_connection != null)
		{
			this._pidSystemObjekt = pidSystemObject;
			this._pidAtg = pidAtg;
			this._pidAsp = pidAsp;
			this._simVar = simVar;

			synchronized (_syncEmpfangBeendet)
			{
				try
				{
					Empfaenger e = new Empfaenger();

					e.startThread();
					if (anmeldenEmpfang())
					{
						_syncEmpfangBeendet.wait();
					}

				}
				catch (InterruptedException e1)
				{
					e1.printStackTrace();
				}
			}
		}

		return _dataEmpfang;
	}

	/**
	 * Fhrt die Anmeldung zum Empfang durch.
	 */
	private boolean anmeldenEmpfang()
	{
		boolean erfolgreich = true;

		if (_connection != null)
		{
			DataModel dataModel = _connection.getDataModel();

			_systemObject = dataModel.getObject(_pidSystemObjekt);
			AttributeGroup _atg = dataModel.getAttributeGroup(_pidAtg);
			Aspect _asp = dataModel.getAspect(_pidAsp);

			if (_systemObject != null)
			{
				if (_atg != null)
				{
					if (_asp != null)
					{
						_dataDescription = new DataDescription(_atg, _asp, _simVar);

						ReceiveOptions receiveOptions = ReceiveOptions.normal();
						ReceiverRole role = ReceiverRole.receiver();

						_connection.subscribeReceiver(this, _systemObject,
								_dataDescription, receiveOptions, role);
					}
					else
					{
						erfolgreich = false;
					}
				}
				else
				{
					erfolgreich = false;
				}
			}
			else
			{
				erfolgreich = false;
			}

			// String out = "";
			// out = out + "SystemObject: " + _systemObject + "\n";
			// out = out + "DataDescription: " + _dataDescription + "\n";
			// out = out + "ReceiveOptions: " + receiveOptions + "\n";
			// out = out + "Role: " + role;
			// System.out.println(out);

		}
		else
		{
			erfolgreich = false;
		}

		return erfolgreich;
	}

	/**
	 * Fhrt die Abmeldung vom Empfang durch.
	 */
	private void abmeldenEmpfang()
	{
		if ((_connection != null) && (_systemObject != null)
				&& (_dataDescription != null))
		{
			_connection.unsubscribeReceiver(this, _systemObject, _dataDescription);
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		for (ResultData resultData : results)
		{
			if (resultData.hasData())
			{
				DataDescription dataDescription = resultData.getDataDescription();
				if (dataDescription.getAttributeGroup().equals(
						_dataDescription.getAttributeGroup()))
				{
					if (dataDescription.getAspect().equals(_dataDescription.getAspect()))
					{
						if (dataDescription.getSimulationVariant() == _dataDescription
								.getSimulationVariant())
						{
							this._dataEmpfang = resultData.getData();
							break;
						}
					}
				}
			}
		}

		synchronized (_syncEmpfangThread)
		{
			_syncEmpfangThread.notify();
		}

		abmeldenEmpfang();
	}

	/**
	 * Empfangs-Hilfklasse, arbeitet mit einem separaten Thread.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class Empfaenger implements Runnable
	{
		public Empfaenger()
		{
		}

		/**
		 * Startet den Thread
		 */
		public void startThread()
		{
			// System.out.println("=> Starte Empfaenger Thread");
			if (!_laufeEmpfangsThread)
			{
				_laufeEmpfangsThread = true;
				Thread empfangsThread = new Thread(this);
				empfangsThread.setName("Empfangs Thread");
				empfangsThread.start();
			}
		}

		/**
		 * Stoppt den Thread
		 */
		public void stopThread()
		{
			if (_laufeEmpfangsThread)
			{
				_laufeEmpfangsThread = false;
			}
		}

		@Override
		public void run()
		{

			while (_laufeEmpfangsThread)
			{
				synchronized (_syncEmpfangThread)
				{
					try
					{
						_syncEmpfangThread.wait();
						synchronized (_syncEmpfangBeendet)
						{
							_syncEmpfangBeendet.notify();
						}
						stopThread();
					}
					catch (InterruptedException e)
					{
						e.printStackTrace();
					}
				}

			}

		}

	}

	/**
	 * Sendet die Daten, die in einer {@link ParameterEditorKapsel} hinterlegt
	 * sind, an den DaV.
	 * 
	 * @param kapsel
	 *          {@link ParameterEditorKapsel} mit den zu versendenden Daten.
	 * @return Liefert <code>true</code>, wenn die Daten erfolgreich versandt
	 *         werden konnten, sonst <code>false</code>.
	 */
	protected boolean sendeDaten(ParameterEditorKapsel kapsel)
	{
		boolean erfolgreich = false;

		if ((kapsel != null) && (_connection != null))
		{
			if (kapsel.getDataSenden() != null)
			{
				erfolgreich = sendeDaten(kapsel.getPidSystemObjekt(), kapsel
						.getPidAtg(), kapsel.getPidAspSenden(), kapsel
						.getSimulationsVariante(), kapsel.getDataSenden());
			}
		}

		return erfolgreich;
	}

	/**
	 * Erzeugt aus einer {@link ParameterEditorKapsel} ein {@link ResultData} zum
	 * Versenden an den DaV, z.B. ber den {@link UrlasserDialog}.
	 * 
	 * @param parameterEditorKapsel
	 *          {@link ParameterEditorKapsel}, aus der ein {@link ResultData}
	 *          erzeugt werden soll. Die Methode
	 *          {@link ParameterEditorKapsel#getDataSenden()} darf kein
	 *          <code>null</code> liefern!
	 * @return Liefert das erzeugte {@link ResultData} zurck.
	 */
	protected ResultData erzeugeResultData(
			ParameterEditorKapsel parameterEditorKapsel)
	{
		ResultData rsd = null;
		if (_connection != null)
		{
			if (parameterEditorKapsel.getDataSenden() != null)
			{
				DataModel dataModel = _connection.getDataModel();
				SystemObject systemObject = dataModel.getObject(parameterEditorKapsel
						.getPidSystemObjekt());

				AttributeGroup atg = dataModel.getAttributeGroup(parameterEditorKapsel
						.getPidAtg());
				Aspect asp = dataModel.getAspect(parameterEditorKapsel
						.getPidAspSenden());
				DataDescription dataDescription = new DataDescription(atg, asp,
						parameterEditorKapsel.getSimulationsVariante());

				rsd = new ResultData(systemObject, dataDescription, new Date()
						.getTime(), parameterEditorKapsel.getDataSenden());
			}
		}

		return rsd;
	}

	/**
	 * Sendet die Daten unter den angegebenen Parametern an den DaV.
	 * 
	 * @param pidSystemObjekt
	 *          Pid des {@link SystemObject}s, an dem die Daten hngen.
	 * @param pidAtg
	 *          Pid der {@link AttributeGroup}, unter der die Anmeldung zum
	 *          Senden erfolgen soll.
	 * @param pidAsp
	 *          Pid des {@link Aspect}s, unter dem die Anmeldung zum Senden
	 *          erfolgen soll.
	 * @param simVar
	 *          Die Simulationsvariante.
	 * @param data
	 *          Die zu sendenden Daten.
	 * @return Liefert <code>true</code> zurck, wenn die Daten erfolgreich
	 *         gesendet werden konnten, sonst <code>false</code>.
	 */
	protected boolean sendeDaten(String pidSystemObjekt, String pidAtg,
			String pidAsp, short simVar, Data data)
	{
		boolean erfolgreich = false;

		if (_connection != null)
		{
			this._pidSystemObjekt = pidSystemObjekt;
			this._pidAtg = pidAtg;
			this._pidAsp = pidAsp;
			this._simVar = simVar;

			DataModel dataModel = _connection.getDataModel();
			_systemObject = dataModel.getObject(_pidSystemObjekt);

			AttributeGroup atg = dataModel.getAttributeGroup(_pidAtg);
			Aspect asp = dataModel.getAspect(_pidAsp);
			_dataDescription = new DataDescription(atg, asp, _simVar);

			ResultData rsd = new ResultData(_systemObject, _dataDescription,
					new Date().getTime(), data);

			anmeldenSenden();

			synchronized (_sendeSteuerung)
			{
				try
				{
					_sendeSteuerung.wait(_sendeSteuerungTimeOut);

					try
					{
						_connection.sendData(rsd);
						erfolgreich = true;
					}
					catch (DataNotSubscribedException e)
					{
						String msg = "Parametereditor: Daten konnten nicht gesendet werden. Es wurde keine Anmeldung zum Senden durchgefhrt.";
						if (_debug)
						{
							System.err.println(msg + "\n\t" + e.toString());
						}
					}
					catch (SendSubscriptionNotConfirmed e)
					{
						String msg = "Parametereditor: Daten konnten nicht gesendet werden. Die Anmeldung zum Senden wurde nicht besttigt.";
						if (_debug)
						{
							System.err.println(msg + "\n\t" + e.toString());
						}
					}

				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}

			abmeldenSenden();
		}

		return erfolgreich;
	}

	/**
	 * Fhrt die Anmeldung zum Senden von Daten durch
	 */
	private void anmeldenSenden()
	{
		SenderRole senderRole = SenderRole.sender();

		try
		{
			_connection.subscribeSender(this, _systemObject, _dataDescription,
					senderRole);
		}
		catch (OneSubscriptionPerSendData e)
		{
			String msg = "Parametereditor: Keine Anmeldung zum Senden mglich.";
			if (_debug)
			{
				System.err.println(msg + "\n\t" + e.toString());
			}
		}

	}

	/**
	 * Fhrt die Abmeldung vom Senden von Daten durch
	 */
	private void abmeldenSenden()
	{
		_connection.unsubscribeSender(this, _systemObject, _dataDescription);

	}

	@Override
	public void dataRequest(SystemObject object, DataDescription dataDescription,
			byte state)
	{
		if (state == ClientSenderInterface.START_SENDING)
		{
			synchronized (_sendeSteuerung)
			{
				_sendeSteuerung.notify();
			}
		}
	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription)
	{
		/*
		 * Liefert der Sendesteuerung true, denn Daten knnen gesendet werden.
		 */
		return true;
	}
}
