/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Collection;
import java.util.HashMap;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnitte;

/**
 * Verwaltet die Instanzen von {@link TSFBereichSchalten}, welche sich gerade in
 * Bearbeitung befinden.<br>
 * Benutzt werden soll diese Klasse vom {@link DialogWeiterschalten} um auch das
 * weitere Schalten eines ganzen Bereichs von Abschnitten zu unterbinden.
 * 
 * @author HofmannJ
 * 
 */
public class TSFBereichSchaltenVerwaltung
{
	private static TSFBereichSchaltenVerwaltung _instance = null;

	/**
	 * Wird bermittelt, wenn ein Bereich zum Schalten hinzugefgt wurde.
	 */
	public static final String PROP_BEREICH_SCHALTEN_ADDED = "BEREICH_SCHALTEN_ADDED";

	/**
	 * Wird bermittelt, wenn ein Bereich zum Schalten entfernt wurde.
	 */
	public static final String PROP_BEREICH_SCHALTEN_REMOVED = "BEREICH_SCHALTEN_REMOVED";

	/**
	 * Fr Listener, die ber hinzugefgte/entfernte Bereiche informiert werden
	 * mchten.
	 */
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * Map mit den Aktiven Prozessen {@link TSFBereichSchalten}, die gerade am
	 * Laufen sind. Die PID entspricht der des jeweiligen Objektes vom Typ
	 * {@link M_TSFAbschnitte}, entsprechend
	 * <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 */
	private HashMap<String, TSFBereichSchalten> _map = new HashMap<String, TSFBereichSchalten>();

	private TSFBereichSchaltenVerwaltung()
	{
	}

	public static TSFBereichSchaltenVerwaltung getInstance()
	{
		if (_instance == null)
		{
			synchronized (TSFBereichSchaltenVerwaltung.class)
			{
				if (_instance == null)
				{
					_instance = new TSFBereichSchaltenVerwaltung();
				}
			}
		}

		return _instance;
	}

	public void putTSFBereichSchalten(TSFBereichSchalten bereichSchalten)
	{
		if ((bereichSchalten != null)
				&& (bereichSchalten.getBereichAbschnitte() != null)
				&& (bereichSchalten.getBereichAbschnitte().getSo() != null))
		{
			synchronized (_map)
			{
				_map.put(bereichSchalten.getBereichAbschnitte().getSo().getPid(),
						bereichSchalten);				
			}
			
			_pcs.firePropertyChange(PROP_BEREICH_SCHALTEN_ADDED, null, null);
		}
	}

	public void removeTSFBereichSchalten(TSFBereichSchalten bereichSchalten)
	{
		if ((bereichSchalten != null)
				&& (bereichSchalten.getBereichAbschnitte() != null)
				&& (bereichSchalten.getBereichAbschnitte().getSo() != null))
		{
			synchronized (_map)
			{
				_map.remove(bereichSchalten.getBereichAbschnitte().getSo().getPid());
			}
		}
		
		_pcs.firePropertyChange(PROP_BEREICH_SCHALTEN_REMOVED, null, null);
	}

	public TSFBereichSchalten[] getTSFAlleBereichSchalten()
	{
		TSFBereichSchalten[] re = null;

		synchronized (_map)
		{
			Collection<TSFBereichSchalten> values = _map.values();
			re = values.toArray(new TSFBereichSchalten[values.size()]);
		}

		return re;
	}

	/**
	 * Liefert {@link TSFBereichSchalten}-Vorgang fr ein Objekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * 
	 * @param pid
	 *          Pid eines Objekts vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return Instanz des {@link TSFBereichSchalten}-Vorgangs oder
	 *         <code>null</code> wenn nicht verfgbar.
	 */
	public TSFBereichSchalten getTsfBereichSchalten(String pid)
	{
		TSFBereichSchalten re = null;

		synchronized (_map)
		{
			re = _map.get(pid);
		}

		return re;
	}

	/**
	 * Prft ob ein Objekt vom Typ <code>typ.tsfAbschnitt</code> gerade in einem
	 * aktiven {@link TSFBereichSchalten}-Prozess befindet oder nicht.
	 * 
	 * @param pid
	 *          PID des zu prfenden Objekts vom Typ <code>typ.tsfAbschnitt</code>
	 *          .
	 * @return Liefert <code>true</code> wenn dieses Objekt sich in einem aktiven
	 *         Prozess befindet, sonst <code>false</code>.
	 */
	public boolean isAbschnittInAktiverBereichsschaltung(String pid)
	{
		boolean re = false;

		TSFBereichSchalten[] aAktiv = getTSFAlleBereichSchalten();
		for (TSFBereichSchalten tsfBereichSchalten : aAktiv)
		{
			M_TSFAbschnitte bereichAbschnitte = tsfBereichSchalten
					.getBereichAbschnitte();
			SystemObject[] aAbs = bereichAbschnitte.getAbschnitte();
			for (SystemObject soAbs : aAbs)
			{
				if (soAbs.getPid().equals(pid))
				{
					re = true;
					break;
				}
			}
		}

		return re;
	}

	/**
	 * Prft ob sich ein Objekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code> gerade in einem aktiven
	 * Schaltungsprozess befindet.
	 * 
	 * @param pid
	 *          PID des Objektes vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return Liefert <code>true</code> wenn sich das Objekt in einer aktiven
	 *         Schaltung befindet, sonst <code>false</code>.
	 */
	public boolean isBereichInAktiverSchaltung(String pid)
	{
		boolean re = false;

		// TSFBereichSchalten[] tsfAlleBereichSchalten =
		// getTSFAlleBereichSchalten();
		// for (TSFBereichSchalten tsfBereichSchalten : tsfAlleBereichSchalten)
		// {
		// if ((tsfBereichSchalten != null)
		// && (tsfBereichSchalten.getBereichAbschnitte() != null)
		// && (tsfBereichSchalten.getBereichAbschnitte().getSo() != null))
		// {
		// if (tsfBereichSchalten.getBereichAbschnitte().getSo().getPid().equals(
		// pid))
		// {
		// re = true;
		// break;
		// }
		// }
		// }

		TSFBereichSchalten tsfBereichSchalten = getTsfBereichSchalten(pid);
		if (tsfBereichSchalten != null)
		{
//			re = true;
			re = tsfBereichSchalten.istJobAktiv();
		}
		

		return re;
	}

	
	public void addListenerBereichschalten(PropertyChangeListener pcl)
	{
		_pcs.addPropertyChangeListener(pcl);
		
		TSFBereichSchalten[] alleBereichSchalten = getTSFAlleBereichSchalten();
		if(alleBereichSchalten.length>0)
		{
			pcl.propertyChange(new PropertyChangeEvent(this, PROP_BEREICH_SCHALTEN_ADDED, null, null));
		}
	}
	
	public void removeListenerBereichschalten(PropertyChangeListener pcl)
	{
		_pcs.removePropertyChangeListener(pcl);
	}
	
	
	
	
}
